/***************************************************************************
                          mosixhistory.cpp  -  description
                             -------------------
    begin                : Sun Sep 23 18:46:39 GMT 2001
    copyright            : (C) 2001 by M. Rechenburg
    email                : rechenburger@netcologne.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qaccel.h>
#include "mosixhistory.h"

#include "fileopen.xpm"
#include "white.xpm"
#include "proc.xpm"
#include "proc_remote.xpm"
#include "ende.xpm"
#include "proc_cant.xpm"
#include "helpicon.xpm"
#include "comment.xpm"
#include "load.xpm"
#include "collector.xpm"
#include "fileopen_hell.xpm"
#include "filefind.xpm"
#include "fileprint.xpm"


MosixhistoryApp::MosixhistoryApp() {

  setCaption(tr("Mosixhistory " VERSION));
  initActions();
  initMenuBar();
}


MosixhistoryApp::~MosixhistoryApp()
{
}


void MosixhistoryApp::initActions(){

	QPixmap white_pix, fresh, end;
	white_pix= QPixmap(white_xpm);
	end = QPixmap(ende);
  QPixmap openIcon, saveIcon, newIcon;
  openIcon = QPixmap(fileopen);
  QPixmap printicon;
  printicon = QPixmap(fileprint);

  fileOpen = new QAction(tr("Open File"), openIcon, tr("&Open..."), 0, this);
  fileOpen->setStatusTip(tr("Opens an mosixcollector logfile"));
  fileOpen->setWhatsThis(tr("Open an\n\nmosixcollector logfile"));
  connect(fileOpen, SIGNAL(activated()), this, SLOT(slotFileOpen()));

  filePrint = new QAction(tr("Print"), printicon, tr("&Print..."), 0, this);
  filePrint->setStatusTip(tr("Print current proc-history"));
  filePrint->setWhatsThis(tr("Print current proc-history"));
  connect(filePrint, SIGNAL(activated()), this, SLOT(printit()));

  fileQuit = new QAction(tr("Exit"), tr("E&xit"), QAccel::stringToKey(tr("Ctrl+Q")), this);
  fileQuit->setStatusTip(tr("Quits the application"));
  fileQuit->setWhatsThis(tr("Exit\n\nQuits the application"));
  connect(fileQuit, SIGNAL(activated()), qApp, SLOT(quit()));

  helpAboutApp = new QAction(tr("About"), tr("&About..."), 0, this);
  helpAboutApp->setStatusTip(tr("About the application"));
  helpAboutApp->setWhatsThis(tr("About\n\nAbout the application"));
  connect(helpAboutApp, SIGNAL(activated()), this, SLOT(slotHelpAbout()));

  helpAboutApp1 = new QAction(tr("Help"), tr("&Help"), 0, this);
  helpAboutApp1->setStatusTip(tr("Help"));
  helpAboutApp1->setWhatsThis(tr("Help"));
  connect(helpAboutApp1, SIGNAL(activated()), this, SLOT(slotHelpAbout1()));

  QSlider_timeline= new QSlider(this,"time-line");
	QSlider_timeline->setGeometry(5,90,340,30);
	QSlider_timeline->setMinimumSize(0,0);
	QSlider_timeline->setTracking(false);
	QSlider_timeline->setValue(0);
	QSlider_timeline->setRange(0,1440);
	QSlider_timeline->setTickInterval (60);
	QSlider_timeline->setTickmarks (QSlider::Below);
	QSlider_timeline->setPageStep(1);
	QSlider_timeline->setOrientation(QSlider::Horizontal);
	QWhatsThis::add(QSlider_timeline,"Use this slider to go on in the time-line of the process-history\n");
	QObject::connect(QSlider_timeline, SIGNAL(valueChanged(int)), this, SLOT(fill_procbox(int)) );

	plus_min= new QLCDNumber(this, "time-line1");
	plus_min->setGeometry(175,125,50,20);
	plus_min->setMinimumSize(0,0);
	plus_min->display("0");
	plus_min->setNumDigits(6);
	plus_min->setSegmentStyle (QLCDNumber::Flat);
	QWhatsThis::add(plus_min ,"The LCD-number displays the minutes from when the\nmosixcollector was started\n");
	QObject::connect(QSlider_timeline, SIGNAL(valueChanged(int)), plus_min, SLOT(display(int)) );

  titel_2= new QLabel(this,"NoName");
  titel_2->setGeometry(75,125,100,20);
  titel_2->setMinimumSize(0,0);
  titel_2->setText("start date/time +");
  titel_2->setMargin(1);
  titel_2->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(titel_2 ,"This box displays when the mosixcollector was started.\n");

  titel_3= new QLabel(this,"NoName");
  titel_3->setGeometry(230,125,40,20);
  titel_3->setMinimumSize(0,0);
  titel_3->setText("min.");
  titel_3->setMargin(1);
  titel_3->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(titel_3 ,"This box displays when the mosixcollector was started.\n");

  titel_4= new QLabel(this,"NoName");
  titel_4->setGeometry(365,75,35,20);
  titel_4->setMinimumSize(0,0);
  titel_4->setText("+24h");
  titel_4->setMargin(1);
  titel_4->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(titel_4 ,"This box displays the end time of the mosixcollector this day.\n");

  titel_5= new QLabel(this,"NoName");
  titel_5->setGeometry(45,75,150,20);
  titel_5->setMinimumSize(0,0);
  titel_5->setText("- <-- TIME-LINE --> +");
  titel_5->setMargin(1);
  titel_5->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(titel_5 ,"Time-line slider to go on in the time of the process-history.\n");

  titel_6= new QLabel(this,"NoName");
  titel_6->setGeometry(5,75,35,20);
  titel_6->setMinimumSize(0,0);
  titel_6->setText("0h");
  titel_6->setMargin(0);
  titel_6->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(titel_6 ,"This box displays the start time of the mosixcollector this day.\n");

  output_label= new QLabel(this,"NoName");
  output_label->setGeometry(50,475,250,25);
  output_label->setMinimumSize(0,0);
  output_label->setText("");
  output_label->setMargin(1);
  output_label->setBackgroundPixmap(QPixmap(white_pix));
  output_label->setAlignment( AlignVCenter | AlignLeft );
  output_label->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	QWhatsThis::add(output_label ,"This box displays the output of your actions.\n");

  output_label1= new QLabel(this,"NoName");
  output_label1->setGeometry(5,475,50,25);
  output_label1->setMinimumSize(0,0);
  output_label1->setText("Output:");
  output_label1->setMargin(1);
  output_label1->setAlignment( AlignVCenter | AlignLeft );
	QWhatsThis::add(output_label1 ,"This box displays the output of your actions.\n");

  fileToolbar = new QToolBar(this, "QWhatsThis");
  fileToolbar->setGeometry(0,0,25,25);
  fileOpen->addTo(fileToolbar);
  fileToolbar->addSeparator();
  filePrint->addTo(fileToolbar);
  fileToolbar->addSeparator();
  QWhatsThis::whatsThisButton(fileToolbar);

}







void MosixhistoryApp::initMenuBar() {

  fileMenu=new QPopupMenu();
  fileOpen->addTo(fileMenu);
  fileMenu->insertSeparator();
  filePrint->addTo(fileMenu);
  fileMenu->insertSeparator();
  fileQuit->addTo(fileMenu);
  helpMenu=new QPopupMenu();
  helpAboutApp1->addTo(helpMenu);
	helpMenu->insertSeparator();
  helpAboutApp->addTo(helpMenu);
  menuBar()->insertItem(tr("&File"), fileMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(tr("&Help"), helpMenu);

}





void MosixhistoryApp::slotFileOpen() {

  QString fileName = QFileDialog::getOpenFileName(0,0,this);
  if (!fileName.isEmpty()) {

		if(fileName.contains("mosix.load", 1)) {
	    doc->load(fileName);
			printf("analyze logfile\n");
			} else {
			printf("not a mosixcollector logfile! (default is /tmp/mosixview_[collect_date]/mosix.load\n");
		  QMessageBox::about(this,tr("not a mosixcollector logfile"),tr("Please load a main mosixcollector history logfile!\n e.g. /tmp/mosixview_[collect_date]/mosix.load") );
		}
	  } else {
   }
}




void MosixhistoryApp::slotHelpAbout() {

  QMessageBox::about(this,tr("About..."), tr("Mosixhistory\nVersion " VERSION "\n(c) 2001 by M. Rechenburg") );
}





void MosixhistoryApp::slotHelpAbout1() {

 	QPixmap white_pix;
	white_pix= QPixmap(white_xpm);
  QPixmap help, com, load_icon, collector_icon, open_icon, finder;
	help = QPixmap(helpicon);
	com = QPixmap(comment);
	load_icon = QPixmap(load);
	collector_icon = QPixmap(collector);
	open_icon = QPixmap(fileopen_hell);
	finder = QPixmap(filefind);

	QTabDialog *tabs = new QTabDialog();
	tabs->setGeometry(200,50,300,200);
	tabs->resize(530, 300);

	QLabel *a = new QLabel(tabs, 0, 0);
	a->setGeometry(5,5,20,20);
	a->resize(200, 200);
	a->setMargin(3);
	a->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	a->setBackgroundPixmap(QPixmap(white_pix));
	a->setText("Help on MOSIX- LOAD, MEM, HISTORY and COLLECTOR\n\n"	\
	"MOSIXLOAD, MOSIXMEM, MOSIXHISTORY and MOSIXCOLLECTOR are \n" \
	"usefull to have a history-view of your MOSIX-cluster.\n" \
	"The MOSIXCOLLECTOR is a daemon running on one cluster-node.\n" \
	"It saves the load+mem of each cluster-member to a directory.\n" \
	"MOSIXLOAD, -MEM and HISTORY analyzes the files in this directory\n" \
	"and produce a graphical overview of the load- and memory history.\n" \
	"All MOSIX-tools can be managed from the main MOSIXVIEW-application.\n" \
	"You can even set checkpoints everytime you want. These checkpoints\n" \
	"are marked in MOSIXLOAD/MOSIXMEM. This help describes how to use these tools.\n" \
	"M. Rechenburg / rechenburger@netcologne.de");

	QLabel *b = new QLabel(tabs, 0, 0);
	b->setGeometry(5,5,20,20);
	b->resize(200, 200);
	b->setMargin(3);
	b->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	b->setBackgroundPixmap(QPixmap(white_pix));
	b->setText("MOSIXCOLLECTOR\n"	\
	"The MOSIXCOLLECTOR is a small daemon started on one cluster-member.\n" \
	"It collects the load/mem/procs of all cluster-nodes to /tmp/mosixview/*\n" \
	"and writes a  PID-file to /tmp/mosixcollector.pid.\n" \
	"mosixcollector -d			// starts the collector as a daemon\n" \
	"mosixcollector -k			// stops the collector\n" \
	"mosixcollector -c			// stops the collector and delete the history-files\n" \
	"mosixcollector -n			// write a checkpoint to the history\n" \
	"mosixcollector -r			// saves the history and start a new one\n" \
	"				// (automatically executed once a day!)\n" \
	"mosixcollector -help		// print out a short help\n" \
	"mosixcollector -h			// print out a short help\n");

	QLabel *c = new QLabel(tabs, 0, 0);
	c->setGeometry(5,5,20,20);
	c->resize(200, 200);
	c->setMargin(3);
	c->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	c->setBackgroundPixmap(QPixmap(white_pix));
	c->setText("MOSIXLOAD\n\n"	\
	"MOSIXLOAD analyzes the data collected by MOSIXCOLLECTOR\n" \
	"in a graphical way. By default it displays the current dataset\n" \
	"from the directory /tmp/mosixview/* but of course you can open older\n" \
	"history logfiles by the filemenu.\n" \
	"MOSIXLOAD displays one whole day of load information from your cluster.\n" \
	"Black lines are values <50, yellow lines are values >50 and the red lines are\n" \
	"values above 80. If there was a checkpoint written you can see it as\n" \
	"a vertical blue line. You can easily get the avarage value of each node\n" \
	"by clicking on the X-button of each node.\n");

	QLabel *d = new QLabel(tabs, 0, 0);
	d->setGeometry(5,5,20,20);
	d->resize(200, 200);
	d->setMargin(3);
	d->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	d->setBackgroundPixmap(QPixmap(white_pix));
	d->setText("MOSIXMEM\n\n"	\
	"MOSIXMEM is similar to MOSIXLOAD\n" \
	"By default it displays the current dataset from the directory\n" \
	"/tmp/mosixview/* and like MOSIXLOAD you can open older history\n" \
	"logfiles by the filemenu, too.\n" \
	"MOSIXMEM displays one whole day of memory information from your cluster.\n" \
	"The black line shows the used memory in percent from the avaiable mem.\n" \
	"You can get the avarage value of used memory by clicking on the\n" \
	"X-button of each node.\n");

	QLabel *e = new QLabel(tabs, 0, 0);
	e->setGeometry(5,5,20,20);
	e->resize(200, 200);
	e->setMargin(3);
	e->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	e->setBackgroundPixmap(QPixmap(white_pix));
	e->setText("MOSIXHISTORY\n\n"	\
	"MOSIXHISTORY is a process-history view.\n" \
	"By default it displays the current dataset from the directory\n" \
	"/tmp/mosixview/* and like MOSIXLOAD (or MOSIXMEM) you can \n" \
	"open older history logfiles by the filemenu, too.\n" \
	"MOSIXHISTORY starts at the start-date of the MOSIXCOLLECTOR\n" \
	"and you can move on in time by the time-line slider.\n" \
	"One mouseclick is similar to one minute.\n" \
	"The processbox displays the processes and where they ran\n" \
	"in dependency of the time (startdate +minutes).\n");

	QLabel *f = new QLabel(tabs, 0, 0);
	f->setGeometry(5,5,20,20);
	f->resize(200, 200);
	f->setMargin(3);
	f->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	f->setBackgroundPixmap(QPixmap(white_pix));
	f->setText("Open older logfiles\n\n"	\
	"The MOSIXCOLLECTOR writes the current dataset of load-informations to\n" \
	"/tmp/mosixview/*  and saves this directory to /tmp/mosixview[date]/*\n" \
	"automatically once a day.\n" \
	"Open the main logfile /tmp/mosixview[date]/mosix.load by the filemenu\n" \
	"and you can analyze the older logfiles.\n" \
	"\n" \
	"You have to open only the main logfile mosix.load for the complet history\n" \
	"information view.\n" \
	"\n");

	QLabel *g = new QLabel(tabs, 0, 0);
	g->setGeometry(5,5,20,20);
	g->resize(200, 200);
	g->setMargin(3);
	g->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	g->setBackgroundPixmap(QPixmap(white_pix));
	g->setText("Comments\n\n"	\
	"If you have more questions on MOSIXLOAD, MOSIXMEM, MOSIXCOLLECTOR\n" \
	"or MOSIXHISTORY you may want to take a look at the MOSIXVIEW-website.\n" \
	"There is a section which describes how to use them.\n" \
	"\n" \
	"http://www.waplocater.de/mosixview/\n" \
	"\n" \
	"Any comments/wanted features/bug-reports are welcome!\n" \
	"Please send them to the MOSIXVIEW-mailinglist or directly to me.\n" \
	"\n" \
	"M. Rechenburg / rechenburger@netcologne.de\n");

	tabs->addTab(a, help,  "general");
	tabs->addTab(b, collector_icon,  "collector");
	tabs->addTab(c, load_icon,  "load-analyzer");
	tabs->addTab(d, load_icon,  "mem-analyzer");
	tabs->addTab(e, finder,  "proc-history");
	tabs->addTab(f, open_icon,  "open logfile");
	tabs->addTab(g, com,  "comments");
	tabs->show();

}





void MosixhistoryApp::neuwin(){

	FILE *zahl2;
	char *cstatus;
	cstatus = new char[4];
	int nummerint=0;
	char *dat1;
	dat1 = new char[50];
	char *dat2;
	dat2 = new char[50];
	char *dat3;
	dat3 = new char[50];
	char *dat4;
	dat4 = new char[50];
	char *dat5;
	dat5 = new char[50];

	QPixmap white_pix;
	white_pix= QPixmap(white_xpm);

	if((zahl2=fopen("/tmp/mosixview/mosix.load", "r")) == NULL) {
		printf("could not open mosix data information file\n");
		strcpy(dat4, "no data");
		} else {
		fscanf(zahl2, "%d", &nummerint);
		fscanf(zahl2, "%s %s %s %s %s", dat1, dat2, dat3, dat4, dat5);
		fclose(zahl2);
	}

  wann= new QLabel(this,"NoName");
  wann->setGeometry(5,125,70,20);
  wann->setMinimumSize(0,0);
  wann->setBackgroundPixmap(QPixmap(white_pix));
  wann->setText(dat4);
  wann->setMargin(0);
  wann->setAlignment( AlignVCenter | AlignCenter );
  wann->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	QWhatsThis::add(wann ,"This box displays when the mosixcollector was started.\n");

	processbox = new QListView(this);
	processbox->setGeometry(5,120,340,330);
	processbox->setMinimumSize(0,0);
	processbox->setAllColumnsShowFocus(1);
	processbox->addColumn("pid");
	processbox->addColumn("n#");
	processbox->addColumn("stat");
	processbox->addColumn("cmdline");
	processbox->addColumn("nice");
	processbox->addColumn("UID");
	processbox->setSorting(2, TRUE);
	processbox->setColumnAlignment (0, AlignLeft);
	processbox->setColumnAlignment (1, AlignLeft);
	processbox->setColumnAlignment (2, AlignLeft);
	processbox->setItemMargin(2);
	processbox->resize(340, 330);
	processbox->setName("/tmp/mosixview/");
	fill_procbox();
	processbox->show();
  QWhatsThis::add(processbox,"This box displays the\nprocesses-history of collector-node.\n");

}






void MosixhistoryApp::neuwin(const char *filename){

	FILE *zahl2;
	char *cstatus;
	cstatus = new char[4];
	int nummerint=0;
	char *dat1;
	dat1 = new char[50];
	char *dat2;
	dat2 = new char[50];
	char *dat3;
	dat3 = new char[50];
	char *dat4;
	dat4 = new char[50];
	char *dat5;
	dat5 = new char[50];
	char *tmp;
	tmp = new char[50];
	char *processbox_name9;
	processbox_name9 = new char[50];
	int laenge9=0;

	QPixmap white_pix;
	white_pix= QPixmap(white_xpm);

	if((zahl2=fopen(filename, "r")) == NULL) {
		printf("could not open mosix data information file\n");
		strcpy(dat4, "no data");
		} else {
		fscanf(zahl2, "%d", &nummerint);
		fscanf(zahl2, "%s %s %s %s %s", dat1, dat2, dat3, dat4, dat5);
		fclose(zahl2);
	}

  wann= new QLabel(this,"NoName");
  wann->setGeometry(5,125,70,20);
  wann->setMinimumSize(0,0);
  wann->setBackgroundPixmap(QPixmap(white_pix));
  wann->setText(dat4);
  wann->setMargin(0);
  wann->setAlignment( AlignVCenter | AlignCenter );
  wann->setFrameStyle( QFrame::Panel | QFrame::Sunken );
	QWhatsThis::add(wann ,"This box displays when the mosixcollector was started.\n");

	laenge9=strlen(filename);
	strcpy(tmp, "");
	strncat(tmp, filename, laenge9-10);
	strcpy(processbox_name9, "");
	strcpy(processbox_name9, tmp);
	strcat(processbox_name9, "\0");

	processbox = new QListView(this);
	processbox->setGeometry(5,120,340,330);
	processbox->setMinimumSize(0,0);
	processbox->setAllColumnsShowFocus(1);
	processbox->addColumn("pid");
	processbox->addColumn("n#");
	processbox->addColumn("stat");
	processbox->addColumn("cmdline");
	processbox->addColumn("nice");
	processbox->addColumn("UID");
	processbox->setSorting(2, TRUE);
	processbox->setColumnAlignment (0, AlignLeft);
	processbox->setColumnAlignment (1, AlignLeft);
	processbox->setColumnAlignment (2, AlignLeft);
	processbox->setItemMargin(2);
	processbox->resize(340, 330);
	processbox->setName(processbox_name9);
	fill_procbox();
	processbox->show();
  QWhatsThis::add(processbox,"This box displays the\nprocesses-history of the collector-node.\n");

	delete [] tmp;
	delete [] processbox_name9;

}







void MosixhistoryApp::fill_procbox() {

	FILE *fp;
  char *cmdline;
	cmdline = new char[1000];
  char *temp;
	temp = new char[1000];
  char *where;
	where = new char[5];
  char *stat;
	stat = new char[1000];
  char *cannot;
	cannot = new char[100];
  char *output;
	output = new char[100];
  char *cproc_nice;
	cproc_nice = new char[6];
  char *line;
	line = new char[80];
  char *userid;
	userid = new char[8];
  char *userid1;
	userid1 = new char[8];
  char *procid;
	procid = new char[8];
  char *fname;
	fname = new char[50];
	int i=0;

 	QPixmap procicon, remoteprocicon, cantprocicon;
	procicon= QPixmap(proc);
	remoteprocicon= QPixmap(proc_remote);
	cantprocicon= QPixmap(proc_cant);
	QPixmap white_pix;
	white_pix= QPixmap(white_xpm);

	strcpy(fname, "");
	strcpy(fname, processbox->name());
	strcat(fname, "/phist/1.procs");

	if (!QFile::exists(fname)) {
		strcpy(output, "");
		strcpy(output, "Could not open data file for reading.");
		printf("Could not open data file for reading.\n");
		} else {
		processbox->clear();
		strcpy(output, "");
		strcpy(output, "Processlist from startdate + 0 min.");
		output_label->setText(output);

		if ((fp=fopen(fname, "r"))!=NULL) {

			while(!feof(fp)) {
				fscanf(fp, "%s %s %s %s %s %s", procid, where, stat, cmdline, cproc_nice, userid);
				i++;
				i1[i] = new QListViewItem(processbox, procid, where, stat, cmdline, cproc_nice, userid);

				if (atoi(where)==0) {
					i1[i]->setPixmap(0, procicon);
		      } else {
					i1[i]->setPixmap(0, remoteprocicon);
				}

			} // while
      fclose(fp);
		} //if
	} //file exists

	delete [] cmdline;
	delete [] temp;
	delete [] where;
	delete [] stat;
	delete [] cannot;
	delete [] procid;
	delete [] output;
	delete [] fname;

}








void MosixhistoryApp::fill_procbox(int time_val) {

	FILE *fp;
  char *cmdline;
	cmdline = new char[1000];
  char *temp;
	temp = new char[1000];
  char *where;
	where = new char[5];
  char *stat;
	stat = new char[1000];
  char *cannot;
	cannot = new char[100];
  char *output;
	output = new char[100];
  char *cproc_nice;
	cproc_nice = new char[6];
  char *line;
	line = new char[80];
  char *userid;
	userid = new char[8];
  char *userid1;
	userid1 = new char[8];
  char *procid;
	procid = new char[8];
  char *tmp;
	tmp = new char[8];
  char *datfile;
	datfile = new char[50];
	int i=0;

 	QPixmap procicon, remoteprocicon, cantprocicon;
	procicon= QPixmap(proc);
	remoteprocicon= QPixmap(proc_remote);
	cantprocicon= QPixmap(proc_cant);
	QPixmap white_pix;
	white_pix= QPixmap(white_xpm);

	strcpy(tmp, "");
	sprintf(tmp, "%d", time_val);
	strcpy(datfile, "");
	strcpy(datfile, processbox->name());
	strcat(datfile, "phist/");
	strcat(datfile, tmp);
	strcat(datfile, ".procs");

	if (!QFile::exists(datfile)) {

//#################################################

		QSlider_timeline->setValue(QSlider_timeline->value()-1);
	  output_label->setText("No more data files! History ends here.");
		printf("No more data files! History ends here.\n");
		} else {
		processbox->clear();
		strcpy(output, "");
		strcpy(output, "Processlist from startdate + ");
		strcat(output, tmp);
		strcat(output, " min.");
		output_label->setText(output);

		if ((fp=fopen(datfile, "r"))!=NULL) {

			while(!feof(fp)) {
				fscanf(fp, "%s %s %s %s %s %s", procid, where, stat, cmdline, cproc_nice, userid);
				i++;
				i1[i] = new QListViewItem(processbox, procid, where, stat, cmdline, cproc_nice, userid);

				if (atoi(where)==0) {
					i1[i]->setPixmap(0, procicon);
	    	  } else {
					i1[i]->setPixmap(0, remoteprocicon);
				}
			} // while
			fclose(fp);
		} //if
	} //file exists

	delete [] cmdline;
	delete [] temp;
	delete [] where;
	delete [] stat;
	delete [] cannot;
	delete [] procid;
	delete [] datfile;
	delete [] output;

}








void MosixhistoryApp::printit() {

  char *cmdline;
	cmdline = new char[1000];
  char *temp;
	temp = new char[1000];
  char *where;
	where = new char[5];
  char *stat;
	stat = new char[1000];
  char *cannot;
	cannot = new char[100];
  char *cproc_nice;
	cproc_nice = new char[6];
  char *procid;
	procid = new char[8];
  char *userid;
	userid = new char[8];
	FILE *zahl3;
  int j=80;
	int pages=1;
	int h=0;
	char *minutes;
	minutes = new char[3];
	char *cpages;
	cpages = new char[3];
	char *tmp;
	tmp = new char[8];
  char *datfile;
	datfile = new char[50];

	QPen pen1(black, 0);
  QPrinter printer;
	printer.setOrientation(QPrinter::Portrait);

  if (printer.setup(this)) {

		if(QSlider_timeline->value()==0) {
			sprintf(tmp, "%d", 1);
      } else {
			sprintf(tmp, "%d", QSlider_timeline->value());
		}
		strcpy(datfile, "");
		strcpy(datfile, processbox->name());
		strcat(datfile, "phist/");
		strcat(datfile, tmp);
		strcat(datfile, ".procs");

		if (!QFile::exists(datfile)) {
			printf("Could not open datfile %s\n", datfile);
			} else {
			QPainter painter;
			painter.begin(&printer);
			painter.setPen(pen1);
			painter.setFont(QFont("Helvetica", 10));
			painter.drawText(200, 25, "1", 1);
			painter.drawText(25, 720, "logfile-date:", 14);
			painter.drawText(120, 720,   wann->text(), strlen(wann->text()));
			painter.drawText(180, 720, "+", 1);
			painter.drawText(190, 720, tmp, strlen(tmp));
			painter.drawText(210, 720, "min.", 5);
			painter.drawText(25, 740, "logfile-name:", 14);
			painter.drawText(120, 740, datfile, strlen(datfile));
			painter.drawText(40, 75, "PID", 4);
			painter.drawText(75, 75, "#n", 6);
			painter.drawText(95, 75, "stat", 5);
			painter.drawText(120, 75, "cmdline", 8);
			painter.drawText(375, 75, "nice", 5);
			painter.drawText(400, 75, "UID", 7);
			painter.setFont(QFont("Helvetica", 6));
			painter.drawText(300, 750, "printed by MOSIXVIEW (rechenburger@netcologne.de)", 50);
			painter.setFont(QFont("Helvetica", 10));

			painter.moveTo(25, 50);
			for(int i=25;i<=450; i++) {
				painter.lineTo(i, 50);
			}

			painter.moveTo(25, 76);
			for(int i=25;i<=450; i++) {
				painter.lineTo(i, 76);
			}

			painter.moveTo(25, 50);
			for(int i=50;i<=700; i++) {
				painter.lineTo(25, i);
			}

			painter.moveTo(25, 700);
			for(int i=25;i<=450; i++) {
				painter.lineTo(i, 700);
			}

			painter.moveTo(450, 50);
			for(int i=50;i<=700; i++) {
				painter.lineTo(450, i);
			}


      if ((zahl3=fopen(datfile, "r"))!=NULL) {
				while(!feof(zahl3)) {
					fscanf(zahl3, "%s %s %s %s %s %s", procid, where, stat, cmdline, cproc_nice, userid);
					h++;
					j=j+10;
					painter.drawText(40, j, procid, strlen(procid));
					painter.drawText(80, j, where, strlen(where));
					painter.drawText(100, j, stat, strlen(stat));
					painter.drawText(120, j, cmdline, strlen(cmdline));
					painter.drawText(380, j, cproc_nice, strlen(cproc_nice));
					painter.drawText(400, j, userid, strlen(userid));

// ########## new page

					if(j==680) {
						pages++;
						printer.newPage();
						sprintf(cpages, "%d", pages);
						painter.setFont(QFont("Helvetica", 10));
						painter.drawText(200, 25, cpages, strlen(cpages));
						painter.drawText(25, 720, "logfile-date:", 14);
						painter.drawText(120, 720,   wann->text(), strlen(wann->text()));
						painter.drawText(180, 720, "+", 1);
						painter.drawText(190, 720, tmp, strlen(tmp));
						painter.drawText(210, 720, "min.", 5);
						painter.drawText(25, 740, "logfile-name:", 14);
						painter.drawText(120, 740, datfile, strlen(datfile));
						painter.drawText(40, 75, "PID", 4);
						painter.drawText(75, 75, "#n", 6);
						painter.drawText(95, 75, "stat", 5);
						painter.drawText(120, 75, "cmdline", 8);
						painter.drawText(375, 75, "nice", 5);
						painter.drawText(400, 75, "UID", 7);
						painter.setFont(QFont("Helvetica", 6));
						painter.drawText(300, 750, "printed by MOSIXVIEW (rechenburger@netcologne.de)", 50);
						painter.setFont(QFont("Helvetica", 10));

						painter.moveTo(25, 50);
						for(int i=25;i<=450; i++) {
							painter.lineTo(i, 50);
						}

						painter.moveTo(25, 76);
						for(int i=25;i<=450; i++) {
							painter.lineTo(i, 76);
						}

						painter.moveTo(25, 50);
						for(int i=25;i<=450; i++) {
							painter.lineTo(i, 50);
						}

						painter.moveTo(25, 50);
						for(int i=50;i<=700; i++) {
							painter.lineTo(25, i);
						}

						painter.moveTo(25, 700);
						for(int i=25;i<=450; i++) {
							painter.lineTo(i, 700);
						}

						painter.moveTo(450, 50);
						for(int i=50;i<=700; i++) {
							painter.lineTo(450, i);
						}
						j=80;		
					}
				} // while
			  painter.end();
				fclose(zahl3);
			} //if
		}
	}

	delete [] temp;
	delete [] minutes;
	delete [] cpages;
	delete [] datfile;
	delete [] tmp;
	delete [] cmdline;
	delete [] where;
	delete [] stat;
	delete [] cannot;
	delete [] procid;
	delete [] userid;


}










